@@ -1,10 +1,14 @@
 #' Zap labels
 #'
+#' @description
 #' Removes labels, leaving unlabelled vectors as is. Use this if you want to
-#' simply drop all labelling from a data frame. Zapping labels from
+#' simply drop all `labels` from a data frame. Zapping labels from
 #' [labelled_spss()] also removes user-defined missing values,
 #' replacing all with `NA`s.
 #'
+#' This function removes value labels; use [zap_label()] to remove variable
+#' labels.
+#'
 #' @param x A vector or data frame
 #' @family zappers
 #' @export
@@ -18,7 +22,7 @@
 #' zap_labels(x2)
 #'
 #' # zap_labels also works with data frames
-#' df <- tibble::data_frame(x1, x2)
+#' df <- tibble::tibble(x1, x2)
 #' df
 #' zap_labels(df)
 zap_labels <- function(x) {
@@ -31,7 +35,7 @@ zap_labels.default <- function(x) {
 }
 
 #' @export
-zap_labels.labelled <- function(x) {
+zap_labels.haven_labelled <- function(x) {
   attr(x, "labels") <- NULL
   class(x) <- NULL
 
@@ -39,7 +43,7 @@ zap_labels.labelled <- function(x) {
 }
 
 #' @export
-zap_labels.labelled_spss <- function(x) {
+zap_labels.haven_labelled_spss <- function(x) {
   x[is.na(x)] <- NA
 
   attr(x, "labels") <- NULL
