@@ -1,6 +1,6 @@
 
 #line 1 "src/spss/readstat_sav_parse.rl"
-
+#include <limits.h>
 #include <stdlib.h>
 #include "../readstat.h"
 #include "../readstat_iconv.h"
@@ -31,7 +31,7 @@ static int count_vars(sav_ctx_t *ctx) {
     spss_varinfo_t *last_info = NULL;
     int var_count = 0;
     for (i=0; i<ctx->var_index; i++) {
-        spss_varinfo_t *info = &ctx->varinfo[i];
+        spss_varinfo_t *info = ctx->varinfo[i];
         if (last_info == NULL || strcmp(info->name, last_info->name) != 0) {
             var_count++;
         }
@@ -46,7 +46,7 @@ static varlookup_t *build_lookup_table(int var_count, sav_ctx_t *ctx) {
     int i;
     spss_varinfo_t *last_info = NULL;
     for (i=0; i<ctx->var_index; i++) {
-        spss_varinfo_t *info = &ctx->varinfo[i];
+        spss_varinfo_t *info = ctx->varinfo[i];
 
         if (last_info == NULL || strcmp(info->name, last_info->name) != 0) {
             varlookup_t *entry = &table[offset++];
@@ -643,6 +643,7 @@ readstat_error_t sav_parse_long_variable_names_record(void *data, int count, sav
                 (readstat_iconv_inbuf_t)&data, &input_len,
                 (char **)&pe, &output_len);
         if (status == (size_t)-1) {
+            free(table);
             free(output_buffer);
             return READSTAT_ERROR_PARSE;
         }
@@ -655,12 +656,12 @@ readstat_error_t sav_parse_long_variable_names_record(void *data, int count, sav
     int cs;
 
     
-#line 659 "src/spss/readstat_sav_parse.c"
+#line 660 "src/spss/readstat_sav_parse.c"
 	{
 	cs = sav_long_variable_parse_start;
 	}
 
-#line 664 "src/spss/readstat_sav_parse.c"
+#line 665 "src/spss/readstat_sav_parse.c"
 	{
 	int _klen;
 	unsigned int _trans;
@@ -735,49 +736,50 @@ _match:
 		switch ( *_acts++ )
 		{
 	case 0:
-#line 104 "src/spss/readstat_sav_parse.rl"
+#line 105 "src/spss/readstat_sav_parse.rl"
 	{
             varlookup_t *found = bsearch(temp_key, table, var_count, sizeof(varlookup_t), &compare_key_varlookup);
             if (found) {
-                memcpy(ctx->varinfo[found->index].longname, temp_val, str_len);
-                ctx->varinfo[found->index].longname[str_len] = '\0';
-            } else if (ctx->error_handler) {
+                spss_varinfo_t *info = ctx->varinfo[found->index];
+                memcpy(info->longname, temp_val, str_len);
+                info->longname[str_len] = '\0';
+            } else if (ctx->handle.error) {
                 snprintf(error_buf, sizeof(error_buf), "Failed to find %s", temp_key);
-                ctx->error_handler(error_buf, ctx->user_ctx);
+                ctx->handle.error(error_buf, ctx->user_ctx);
             }
         }
 	break;
 	case 1:
-#line 115 "src/spss/readstat_sav_parse.rl"
+#line 117 "src/spss/readstat_sav_parse.rl"
 	{
             memcpy(temp_key, str_start, str_len);
             temp_key[str_len] = '\0';
         }
 	break;
 	case 2:
-#line 120 "src/spss/readstat_sav_parse.rl"
+#line 122 "src/spss/readstat_sav_parse.rl"
 	{
             memcpy(temp_val, str_start, str_len);
             temp_val[str_len] = '\0';
         }
 	break;
 	case 3:
-#line 131 "src/spss/readstat_sav_parse.rl"
+#line 133 "src/spss/readstat_sav_parse.rl"
 	{ str_start = p; }
 	break;
 	case 4:
-#line 131 "src/spss/readstat_sav_parse.rl"
+#line 133 "src/spss/readstat_sav_parse.rl"
 	{ str_len = p - str_start; }
 	break;
 	case 5:
-#line 133 "src/spss/readstat_sav_parse.rl"
+#line 135 "src/spss/readstat_sav_parse.rl"
 	{ str_start = p; }
 	break;
 	case 6:
-#line 133 "src/spss/readstat_sav_parse.rl"
+#line 135 "src/spss/readstat_sav_parse.rl"
 	{ str_len = p - str_start; }
 	break;
-#line 781 "src/spss/readstat_sav_parse.c"
+#line 783 "src/spss/readstat_sav_parse.c"
 		}
 	}
 
@@ -794,30 +796,31 @@ _again:
 	while ( __nacts-- > 0 ) {
 		switch ( *__acts++ ) {
 	case 0:
-#line 104 "src/spss/readstat_sav_parse.rl"
+#line 105 "src/spss/readstat_sav_parse.rl"
 	{
             varlookup_t *found = bsearch(temp_key, table, var_count, sizeof(varlookup_t), &compare_key_varlookup);
             if (found) {
-                memcpy(ctx->varinfo[found->index].longname, temp_val, str_len);
-                ctx->varinfo[found->index].longname[str_len] = '\0';
-            } else if (ctx->error_handler) {
+                spss_varinfo_t *info = ctx->varinfo[found->index];
+                memcpy(info->longname, temp_val, str_len);
+                info->longname[str_len] = '\0';
+            } else if (ctx->handle.error) {
                 snprintf(error_buf, sizeof(error_buf), "Failed to find %s", temp_key);
-                ctx->error_handler(error_buf, ctx->user_ctx);
+                ctx->handle.error(error_buf, ctx->user_ctx);
             }
         }
 	break;
 	case 2:
-#line 120 "src/spss/readstat_sav_parse.rl"
+#line 122 "src/spss/readstat_sav_parse.rl"
 	{
             memcpy(temp_val, str_start, str_len);
             temp_val[str_len] = '\0';
         }
 	break;
 	case 6:
-#line 133 "src/spss/readstat_sav_parse.rl"
+#line 135 "src/spss/readstat_sav_parse.rl"
 	{ str_len = p - str_start; }
 	break;
-#line 821 "src/spss/readstat_sav_parse.c"
+#line 824 "src/spss/readstat_sav_parse.c"
 		}
 	}
 	}
@@ -825,14 +828,14 @@ _again:
 	_out: {}
 	}
 
-#line 141 "src/spss/readstat_sav_parse.rl"
+#line 143 "src/spss/readstat_sav_parse.rl"
 
 
     if (cs < 227|| p != pe) {
-        if (ctx->error_handler) {
+        if (ctx->handle.error) {
             snprintf(error_buf, sizeof(error_buf), "Error parsing string \"%.*s\" around byte #%ld/%d, character %c", 
                     count, (char *)data, (long)(p - c_data), count, *p);
-            ctx->error_handler(error_buf, ctx->user_ctx);
+            ctx->handle.error(error_buf, ctx->user_ctx);
         }
         retval = READSTAT_ERROR_PARSE;
     }
@@ -849,7 +852,7 @@ _again:
 }
 
 
-#line 853 "src/spss/readstat_sav_parse.c"
+#line 856 "src/spss/readstat_sav_parse.c"
 static const char _sav_very_long_string_parse_actions[] = {
 	0, 1, 0, 1, 2, 1, 3, 2, 
 	4, 1, 2, 5, 2
@@ -957,7 +960,7 @@ static const int sav_very_long_string_parse_start = 1;
 static const int sav_very_long_string_parse_en_main = 1;
 
 
-#line 167 "src/spss/readstat_sav_parse.rl"
+#line 169 "src/spss/readstat_sav_parse.rl"
 
 
 readstat_error_t sav_parse_very_long_string_record(void *data, int count, sav_ctx_t *ctx) {
@@ -966,18 +969,18 @@ readstat_error_t sav_parse_very_long_string_record(void *data, int count, sav_ct
     readstat_error_t retval = READSTAT_OK;
 
     char temp_key[8*4+1];
-    int temp_val = 0;
+    unsigned int temp_val = 0;
     unsigned char *str_start = NULL;
     size_t str_len = 0;
 
     size_t error_buf_len = 1024 + count;
-    char *error_buf = readstat_malloc(error_buf_len);
+    char *error_buf = NULL;
     unsigned char *p = NULL;
     unsigned char *pe = NULL;
 
     unsigned char *output_buffer = NULL;
-
-    varlookup_t *table = build_lookup_table(var_count, ctx);
+    varlookup_t *table = NULL;
+    int cs;
 
     if (ctx->converter) {
         size_t input_len = count;
@@ -996,16 +999,17 @@ readstat_error_t sav_parse_very_long_string_record(void *data, int count, sav_ct
         p = c_data;
         pe = c_data + count;
     }
-    
-    int cs;
+
+    error_buf = readstat_malloc(error_buf_len);
+    table = build_lookup_table(var_count, ctx);
     
     
-#line 1004 "src/spss/readstat_sav_parse.c"
+#line 1008 "src/spss/readstat_sav_parse.c"
 	{
 	cs = sav_very_long_string_parse_start;
 	}
 
-#line 1009 "src/spss/readstat_sav_parse.c"
+#line 1013 "src/spss/readstat_sav_parse.c"
 	{
 	int _klen;
 	unsigned int _trans;
@@ -1080,42 +1084,47 @@ _match:
 		switch ( *_acts++ )
 		{
 	case 0:
-#line 209 "src/spss/readstat_sav_parse.rl"
+#line 212 "src/spss/readstat_sav_parse.rl"
 	{
             varlookup_t *found = bsearch(temp_key, table, var_count, sizeof(varlookup_t), &compare_key_varlookup);
             if (found) {
-                ctx->varinfo[found->index].string_length = temp_val;
+                ctx->varinfo[found->index]->string_length = temp_val;
             }
         }
 	break;
 	case 1:
-#line 216 "src/spss/readstat_sav_parse.rl"
+#line 219 "src/spss/readstat_sav_parse.rl"
 	{
             memcpy(temp_key, str_start, str_len);
             temp_key[str_len] = '\0';
         }
 	break;
 	case 2:
-#line 221 "src/spss/readstat_sav_parse.rl"
+#line 224 "src/spss/readstat_sav_parse.rl"
 	{
-            if ((*p) != '\0') { 
-                temp_val = 10 * temp_val + ((*p) - '0'); 
+            if ((*p) != '\0') {
+                unsigned char digit = (*p) - '0';
+                if (temp_val <= (UINT_MAX - digit) / 10) {
+                    temp_val = 10 * temp_val + digit;
+                } else {
+                    {p++; goto _out; }
+                }
             }
         }
 	break;
 	case 3:
-#line 233 "src/spss/readstat_sav_parse.rl"
+#line 241 "src/spss/readstat_sav_parse.rl"
 	{ str_start = p; }
 	break;
 	case 4:
-#line 233 "src/spss/readstat_sav_parse.rl"
+#line 241 "src/spss/readstat_sav_parse.rl"
 	{ str_len = p - str_start; }
 	break;
 	case 5:
-#line 235 "src/spss/readstat_sav_parse.rl"
+#line 243 "src/spss/readstat_sav_parse.rl"
 	{ temp_val = 0; }
 	break;
-#line 1119 "src/spss/readstat_sav_parse.c"
+#line 1128 "src/spss/readstat_sav_parse.c"
 		}
 	}
 
@@ -1128,14 +1137,14 @@ _again:
 	_out: {}
 	}
 
-#line 243 "src/spss/readstat_sav_parse.rl"
+#line 251 "src/spss/readstat_sav_parse.rl"
 
     
     if (cs < 36 || p != pe) {
-        if (ctx->error_handler) {
+        if (ctx->handle.error) {
             snprintf(error_buf, error_buf_len, "Parsed %ld of %ld bytes. Remaining bytes: %.*s",
                     (long)(p - c_data), (long)(pe - c_data), (int)(pe - p), p);
-            ctx->error_handler(error_buf, ctx->user_ctx);
+            ctx->handle.error(error_buf, ctx->user_ctx);
         }
         retval = READSTAT_ERROR_PARSE;
     }
